@@ -1,16 +1,16 @@
-//          Copyright Naoki Shibata 2010 - 2019.
+//   Copyright Naoki Shibata and contributors 2010 - 2021.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
 
-#if CONFIG == 140 || CONFIG == 141
+#if CONFIG == 140 || CONFIG == 141 || CONFIG == 150 || CONFIG == 151
 
 #if !defined(__VX__) && !defined(SLEEF_GENHEADER)
 #error This helper is for IBM s390x.
 #endif
 
 #if __ARCH__ < 12 && !defined(SLEEF_GENHEADER)
-#error Please specify -march=z14.
+#error Please specify -march=z14 or higher.
 #endif
 
 #else
@@ -31,7 +31,7 @@
 #define VECTLENSP (1 << LOG2VECTLENSP)
 //@#define VECTLENSP (1 << LOG2VECTLENSP)
 
-#if CONFIG == 140
+#if CONFIG == 140 || CONFIG == 150
 #define ENABLE_FMA_DP
 //@#define ENABLE_FMA_DP
 #define ENABLE_FMA_SP
@@ -63,6 +63,15 @@ typedef __vector int vint;
 typedef __vector float vfloat;
 typedef __vector int vint2;
 
+typedef __vector long long vint64;
+typedef __vector unsigned long long vuint64;
+
+typedef struct {
+  vmask x, y;
+} vquad;
+
+typedef vquad vargquad;
+
 //
 
 #if !defined(SLEEF_GENHEADER)
@@ -74,7 +83,12 @@ static INLINE int vavailability_i(int n) {
   return 0;
 }
 
-#define ISANAME "ZVECTOR2"
+#if CONFIG == 140 || CONFIG == 141
+#define ISANAME "VXE"
+#else
+#define ISANAME "VXE2"
+#endif
+
 #define DFTPRIORITY 14
 
 #endif // #if !defined(SLEEF_GENHEADER)
@@ -106,15 +120,14 @@ static INLINE void vstoreu_v_p_vf(float *p, vfloat v) { p[0] = v[0]; p[1] = v[1]
 static INLINE void vscatter2_v_p_i_i_vd(double *p, int offset, int step, vdouble v) { vstore_v_p_vd((double *)(&p[2*offset]), v); }
 
 static INLINE vdouble vgather_vd_p_vi(const double *p, vint vi) {
-  int a[VECTLENDP];
   return ((vdouble) { p[vi[0]], p[vi[1]] });
 }
 
 static INLINE vfloat vgather_vf_p_vi2(const float *p, vint2 vi2) {
-  int a[VECTLENSP];
   return ((vfloat) { p[vi2[0]], p[vi2[1]], p[vi2[2]], p[vi2[3]] });
 }
 
+static INLINE vopmask vcast_vo_i(int i) { return (vopmask) { i ? (long long)-1 : 0, i ? (long long)-1 : 0 }; }
 static INLINE vint vcast_vi_i(int i) { return (vint) { i, i }; }
 static INLINE vint2 vcast_vi2_i(int i) { return (vint2) { i, i, i, i }; }
 static INLINE vfloat vcast_vf_f(float f) { return (vfloat) { f, f, f, f }; }
@@ -134,8 +147,6 @@ static INLINE vint2 vtruncate_vi2_vf(vfloat vf) { return (vint) { vf[0], vf[1],
 
 static INLINE vmask vreinterpret_vm_vd(vdouble vd) { return (vmask)vd; }
 static INLINE vdouble vreinterpret_vd_vm(vmask vm) { return (vdouble)vm; }
-static INLINE vint2 vreinterpret_vi2_vd(vdouble vd) { return (vint2)vd; }
-static INLINE vdouble vreinterpret_vd_vi2(vint2 vi) { return (vdouble)vi; }
 
 static INLINE vmask vreinterpret_vm_vf(vfloat vf) { return (vmask)vf; }
 static INLINE vfloat vreinterpret_vf_vm(vmask vm) { return (vfloat)vm; }
@@ -180,15 +191,18 @@ static INLINE vdouble vsel_vd_vo_vd_vd(vopmask o, vdouble x, vdouble y) { return
 static INLINE vfloat vsel_vf_vo_vf_vf(vopmask o, vfloat x, vfloat y) { return vec_sel(y, x, (__vector unsigned int)o); }
 static INLINE vint2 vsel_vi2_vo_vi2_vi2(vopmask o, vint2 x, vint2 y) { return vec_sel(y, x, (__vector unsigned int)o); }
 
-static INLINE int vtestallones_i_vo64(vopmask g) { return g[0] == 0xffffffffffffffffLL && g[1] == 0xffffffffffffffffLL; }
-static INLINE int vtestallones_i_vo32(vopmask g) { return g[0] == 0xffffffffffffffffLL && g[1] == 0xffffffffffffffffLL; }
+static INLINE int vtestallones_i_vo32(vopmask g) { return vec_all_ne((vint2)g, (vint2 ) { 0, 0, 0, 0 }); }
+static INLINE int vtestallones_i_vo64(vopmask g) { return vec_all_ne((__vector unsigned long long)g, (__vector unsigned long long) { 0, 0 }); }
 
 static INLINE vopmask vcast_vo32_vo64(vopmask g) { return (vopmask)(vint) { g[0] != 0 ? -1 : 0, g[1] != 0 ? -1 : 0, 0, 0 }; }
 static INLINE vopmask vcast_vo64_vo32(vopmask g) { return (vopmask) { ((vint)g)[0] != 0 ? 0xffffffffffffffffLL : 0, ((vint)g)[1] != 0 ? 0xffffffffffffffffLL : 0 }; }
 
 static INLINE vmask vcast_vm_i_i(int h, int l) { return (vmask)(vint){ h, l, h, l }; }
-static INLINE vint2 vcastu_vi2_vi(vint vi) { return (vint2){ vi[0], 0, vi[1], 0 }; }
-static INLINE vint vcastu_vi_vi2(vint2 vi2) { return (vint){ vi2[0], vi2[2] }; }
+static INLINE vmask vcast_vm_i64(int64_t i) { return (vmask)(vint64){ i, i }; }
+static INLINE vmask vcast_vm_u64(uint64_t i) { return (vmask)(vuint64){ i, i }; }
+
+static INLINE vmask vcastu_vm_vi(vint vi) { return (vmask)(vint2){ vi[0], 0, vi[1], 0 }; }
+static INLINE vint vcastu_vi_vm(vmask vi2) { return (vint){ vi2[0] >> 32, vi2[1] >> 32 }; }
 
 static INLINE vint vreinterpretFirstHalf_vi_vi2(vint2 vi2) { return (vint){ vi2[0], vi2[1] }; }
 static INLINE vint2 vreinterpretFirstHalf_vi2_vi(vint vi) { return (vint2){ vi[0], vi[1], 0, 0 }; }
@@ -197,7 +211,6 @@ static INLINE vdouble vrev21_vd_vd(vdouble vd) { return (vdouble) { vd[1], vd[0]
 static INLINE vdouble vreva2_vd_vd(vdouble vd) { return vd; }
 static INLINE vfloat vrev21_vf_vf(vfloat vd) { return (vfloat) { vd[1], vd[0], vd[3], vd[2] }; }
 static INLINE vfloat vreva2_vf_vf(vfloat vd) { return (vfloat) { vd[2], vd[3], vd[0], vd[1] }; }
-static INLINE vint2 vrev21_vi2_vi2(vint2 i) { return vreinterpret_vi2_vf(vrev21_vf_vf(vreinterpret_vf_vi2(i))); }
 
 static INLINE vopmask veq64_vo_vm_vm(vmask x, vmask y) {
   return (vopmask) { x[0] == y[0] ? 0xffffffffffffffffLL : 0, x[1] == y[1] ? 0xffffffffffffffffLL : 0 };
@@ -224,7 +237,7 @@ static INLINE vfloat vnegpos_vf_vf(vfloat d) { return vreinterpret_vf_vm(vxor_vm
 static INLINE vdouble vabs_vd_vd(vdouble d) { return vec_abs(d); }
 static INLINE vdouble vsubadd_vd_vd_vd(vdouble x, vdouble y) { return vadd_vd_vd_vd(x, vnegpos_vd_vd(y)); }
 
-#if CONFIG == 140
+#if CONFIG == 140 || CONFIG == 150
 static INLINE vdouble vmla_vd_vd_vd_vd(vdouble x, vdouble y, vdouble z) { return vec_madd(x, y, z); }
 static INLINE vdouble vmlapn_vd_vd_vd_vd(vdouble x, vdouble y, vdouble z) { return vec_msub(x, y, z); }
 static INLINE vdouble vmlanp_vd_vd_vd_vd(vdouble x, vdouble y, vdouble z) { return vec_nmsub(x, y, z); }
@@ -242,7 +255,7 @@ static INLINE vdouble vfmann_vd_vd_vd_vd(vdouble x, vdouble y, vdouble z) { retu
 
 static INLINE vfloat vsubadd_vf_vf_vf(vfloat x, vfloat y) { return vadd_vf_vf_vf(x, vnegpos_vf_vf(y)); }
 
-#if CONFIG == 140
+#if CONFIG == 140 || CONFIG == 150
 static INLINE vfloat vmla_vf_vf_vf_vf  (vfloat x, vfloat y, vfloat z) { return __builtin_s390_vfmasb(x, y, z); }
 static INLINE vfloat vmlanp_vf_vf_vf_vf(vfloat x, vfloat y, vfloat z) { return vec_nmsub(x, y, z); }
 static INLINE vfloat vmlapn_vf_vf_vf_vf(vfloat x, vfloat y, vfloat z) { return __builtin_s390_vfmssb(x, y, z); }
@@ -394,3 +407,56 @@ static INLINE vint2 vrint_vi2_vf(vfloat vf) {
   vf = vrint_vf_vf(vf);
   return (vint) { vf[0], vf[1], vf[2], vf[3] };
 }
+
+//
+
+static vquad loadu_vq_p(void *p) {
+  vquad vq;
+  memcpy(&vq, p, VECTLENDP * 16);
+  return vq;
+}
+
+static INLINE vquad cast_vq_aq(vargquad aq) {
+  vquad m = { aq.y, aq.x };
+  return m;
+}
+static INLINE vargquad cast_aq_vq(vquad vq) {
+  vargquad a = { vq.y, vq.x };
+  return a;
+}
+
+static INLINE int vtestallzeros_i_vo64(vopmask g) {
+  return vec_all_eq((__vector signed long long)g, (__vector signed long long){ 0, 0 });
+}
+
+static INLINE vmask vsel_vm_vo64_vm_vm(vopmask o, vmask x, vmask y) {
+  return (vmask)vec_sel((__vector signed long long)y, (__vector signed long long)x, (__vector __bool long long)o);
+}
+
+static INLINE vmask vsub64_vm_vm_vm(vmask x, vmask y) {
+  return (vmask)((__vector signed long long)x - (__vector signed long long)y);
+}
+
+static INLINE vmask vneg64_vm_vm(vmask x) {
+  return (vmask)((__vector signed long long) {0, 0} - (__vector signed long long)x);
+}
+
+static INLINE vopmask vgt64_vo_vm_vm(vmask x, vmask y) {
+  return (vopmask)vec_cmpgt((__vector signed long long)x, (__vector signed long long)y);
+}
+
+#define vsll64_vm_vm_i(x, c) ((vmask)((__vector unsigned long long)x << (__vector unsigned long long) { c, c }))
+#define vsrl64_vm_vm_i(x, c) ((vmask)((__vector unsigned long long)x >> (__vector unsigned long long) { c, c }))
+
+static INLINE vint vcast_vi_vm(vmask vm) {
+  return (vint) { vm[0], vm[1] };
+}
+
+static INLINE vmask vcast_vm_vi(vint vi) {
+  return (vmask) (__vector signed long long) { vi[0], vi[1] };
+}
+
+static INLINE vmask vreinterpret_vm_vi64(vint64 v) { return (vmask)v; }
+static INLINE vint64 vreinterpret_vi64_vm(vmask m) { return (vint64)m; }
+static INLINE vmask vreinterpret_vm_vu64(vuint64 v) { return (vmask)v; }
+static INLINE vuint64 vreinterpret_vu64_vm(vmask m) { return (vuint64)m; }
