@@ -1,4 +1,4 @@
-//   Copyright Naoki Shibata and contributors 2010 - 2020.
+//   Copyright Naoki Shibata and contributors 2010 - 2021.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -68,50 +68,6 @@ typedef Sleef_float2 sc_t;
 #define BUTB butb_float
 #define SINCOSPI Sleef_sincospif_u05
 #include "dispatchsp.h"
-#elif BASETYPEID == 3
-typedef long double real;
-typedef Sleef_longdouble2 sc_t;
-#define BASETYPESTRING "long double"
-#define MAGIC 0x14142135
-#define MAGIC2D 0x26457513
-#define INIT SleefDFT_longdouble_init1d
-#define EXECUTE SleefDFT_longdouble_execute
-#define INIT2D SleefDFT_longdouble_init2d
-#define CTBL ctbl_longdouble
-#define REALSUB0 realSub0_longdouble
-#define REALSUB1 realSub1_longdouble
-#define GETINT getInt_longdouble
-#define GETPTR getPtr_longdouble
-#define DFTF dftf_longdouble
-#define DFTB dftb_longdouble
-#define TBUTF tbutf_longdouble
-#define TBUTB tbutb_longdouble
-#define BUTF butf_longdouble
-#define BUTB butb_longdouble
-#define SINCOSPI Sleef_sincospil_u05
-#include "dispatchld.h"
-#elif BASETYPEID == 4
-typedef Sleef_quad real;
-typedef Sleef_quad2 sc_t;
-#define BASETYPESTRING "Sleef_quad"
-#define MAGIC 0x33166247
-#define MAGIC2D 0x36055512
-#define INIT SleefDFT_quad_init1d
-#define EXECUTE SleefDFT_quad_execute
-#define INIT2D SleefDFT_quad_init2d
-#define CTBL ctbl_Sleef_quad
-#define REALSUB0 realSub0_Sleef_quad
-#define REALSUB1 realSub1_Sleef_quad
-#define GETINT getInt_Sleef_quad
-#define GETPTR getPtr_Sleef_quad
-#define DFTF dftf_Sleef_quad
-#define DFTB dftb_Sleef_quad
-#define TBUTF tbutf_Sleef_quad
-#define TBUTB tbutb_Sleef_quad
-#define BUTF butf_Sleef_quad
-#define BUTB butb_Sleef_quad
-#define SINCOSPI Sleef_sincospiq_u05
-#include "dispatchqp.h"
 #else
 #error No BASETYPEID specified
 #endif
@@ -121,47 +77,6 @@ typedef Sleef_quad2 sc_t;
 
 //
 
-#if BASETYPEID == 4
-real CTBL[] = {
-  0.7071067811865475243818940365159164684883Q, -0.7071067811865475243818940365159164684883Q,
-  0.9238795325112867561014214079495587839119Q, -0.382683432365089771723257530688933059082Q,
-  0.382683432365089771723257530688933059082Q, -0.9238795325112867561014214079495587839119Q,
-#if MAXBUTWIDTH >= 5
-  0.9807852804032304491190993878113602022495Q, -0.1950903220161282678433729148581576851029Q,
-  0.5555702330196022247573058028269343822103Q, -0.8314696123025452370808655033762590846891Q,
-  0.8314696123025452370808655033762590846891Q, -0.5555702330196022247573058028269343822103Q,
-  0.1950903220161282678433729148581576851029Q, -0.9807852804032304491190993878113602022495Q,
-#endif
-#if MAXBUTWIDTH >= 6
-  0.9951847266721968862310254699821143731242Q, -0.09801714032956060199569840382660679267701Q,
-  0.6343932841636454982026105398063009488396Q, -0.7730104533627369607965383602188325085081Q,
-  0.881921264348355029715105513066220055407Q, -0.4713967368259976485449225247492677226546Q,
-  0.2902846772544623676448431737195932100803Q, -0.9569403357322088649310892760624369657307Q,
-  0.9569403357322088649310892760624369657307Q, -0.2902846772544623676448431737195932100803Q,
-  0.4713967368259976485449225247492677226546Q, -0.881921264348355029715105513066220055407Q,
-  0.7730104533627369607965383602188325085081Q, -0.6343932841636454982026105398063009488396Q,
-  0.09801714032956060199569840382660679267701Q, -0.9951847266721968862310254699821143731242Q,
-#endif
-#if MAXBUTWIDTH >= 7
-  0.9987954562051723927007702841240899260811Q, -0.04906767432741801425355085940205324135377Q,
-  0.6715589548470184006194634573905233310143Q, -0.7409511253549590911932944126139233276263Q,
-  0.9039892931234433315823215138173907234886Q, -0.427555093430282094315230886905077056781Q,
-  0.336889853392220050702686798271834334173Q, -0.9415440651830207783906830087961026265475Q,
-  0.9700312531945439926159106824865574481009Q, -0.2429801799032638899447731489766866275204Q,
-  0.5141027441932217266072797923204262815489Q, -0.8577286100002720698929313536407192941624Q,
-  0.8032075314806449097991200569701675249235Q, -0.5956993044924333434615715265891822127742Q,
-  0.1467304744553617516588479505190711904561Q, -0.9891765099647809734561415551112872890371Q,
-  0.9891765099647809734561415551112872890371Q, -0.1467304744553617516588479505190711904561Q,
-  0.5956993044924333434615715265891822127742Q, -0.8032075314806449097991200569701675249235Q,
-  0.8577286100002720698929313536407192941624Q, -0.5141027441932217266072797923204262815489Q,
-  0.2429801799032638899447731489766866275204Q, -0.9700312531945439926159106824865574481009Q,
-  0.9415440651830207783906830087961026265475Q, -0.336889853392220050702686798271834334173Q,
-  0.427555093430282094315230886905077056781Q, -0.9039892931234433315823215138173907234886Q,
-  0.7409511253549590911932944126139233276263Q, -0.6715589548470184006194634573905233310143Q,
-  0.04906767432741801425355085940205324135377Q, -0.9987954562051723927007702841240899260811Q,
-#endif
-};
-#else
 real CTBL[] = {
   0.7071067811865475243818940365159164684883L, -0.7071067811865475243818940365159164684883L,
   0.9238795325112867561014214079495587839119L, -0.382683432365089771723257530688933059082L,
@@ -201,7 +116,6 @@ real CTBL[] = {
   0.04906767432741801425355085940205324135377L, -0.9987954562051723927007702841240899260811L,
 #endif
 };
-#endif
 
 #ifndef ENABLE_STREAM
 #error ENABLE_STREAM not defined
@@ -215,13 +129,22 @@ extern int planFilePathSet;
 
 // Utility functions
 
+#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
 static jmp_buf sigjmp;
-static void sighandler(int signum) { longjmp(sigjmp, 1); }
+#define SETJMP(x) setjmp(x)
+#define LONGJMP longjmp
+#else
+static sigjmp_buf sigjmp;
+#define SETJMP(x) sigsetjmp(x, 1)
+#define LONGJMP siglongjmp
+#endif
+
+static void sighandler(int signum) { LONGJMP(sigjmp, 1); }
 
 static int checkISAAvailability(int isa) {
   signal(SIGILL, sighandler);
 
-  if (setjmp(sigjmp) == 0) {
+  if (SETJMP(sigjmp) == 0) {
     int ret = GETINT[isa] != NULL && (*GETINT[isa])(BASETYPEID);
     signal(SIGILL, SIG_DFL);
     return ret;
@@ -306,19 +229,6 @@ static void dispatch(SleefDFT *p, const int N, real *d, const real *s, const int
       *(element_t *)&row[x2].r[y2*2+0] = r;				\
     }} while(0)
 
-#if defined(__zarch__) && defined(__GNUC__) && !defined(__clang__)
-// This is a workaround of a bug in gcc on s390
-static void transpose(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(256) s, const int log2n, const int log2m) {
-  for(int y=0;y<(1 << log2n);y++) {
-    for(int x=0;x<(1 << log2m);x++) {
-      real r0 = s[((y << log2m)+x)*2+0];
-      real r1 = s[((y << log2m)+x)*2+1];
-      d[((x << log2n)+y)*2+0] = r0;
-      d[((x << log2n)+y)*2+1] = r1;
-    }
-  }
-}
-#else
 static void transpose(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(256) s, const int log2n, const int log2m) {
   if (log2n < LOG2BS || log2m < LOG2BS) {
     for(int y=0;y<(1 << log2n);y++) {
@@ -369,22 +279,8 @@ static void transpose(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(256)
     }
   }
 }
-#endif // #if defined(__zarch__) && defined(__GNUC__) && !defined(__clang__)
 
 #ifdef _OPENMP
-#if defined(__zarch__) && defined(__GNUC__) && !defined(__clang__)
-// This is a workaround of a bug in gcc on s390
-static void transposeMT(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(256) s, int log2n, int log2m) {
-  for(int y=0;y<(1 << log2n);y++) {
-    for(int x=0;x<(1 << log2m);x++) {
-      real r0 = s[((y << log2m)+x)*2+0];
-      real r1 = s[((y << log2m)+x)*2+1];
-      d[((x << log2n)+y)*2+0] = r0;
-      d[((x << log2n)+y)*2+1] = r1;
-    }
-  }
-}
-#else
 static void transposeMT(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(256) s, int log2n, int log2m) {
   if (log2n < LOG2BS || log2m < LOG2BS) {
     for(int y=0;y<(1 << log2n);y++) {
@@ -437,7 +333,6 @@ static void transposeMT(real *RESTRICT ALIGNED(256) d, real *RESTRICT ALIGNED(25
     }
   }
 }
-#endif // #if defined(__zarch__) && defined(__GNUC__) && !defined(__clang__)
 #endif // #ifdef _OPENMP
 
 // Table generator
@@ -1254,7 +1149,7 @@ EXPORT SleefDFT *INIT(uint32_t n, const real *in, real *out, uint64_t mode) {
   p->isa = -1;
 
   for(int i=0;i<ISAMAX;i++) {
-    if (checkISAAvailability(i) && bestPriority < (*GETINT[i])(GETINT_DFTPRIORITY) && n >= (*GETINT[i])(GETINT_VECWIDTH) * (*GETINT[i])(GETINT_VECWIDTH)) {
+    if (checkISAAvailability(i) && bestPriority < (*GETINT[i])(GETINT_DFTPRIORITY) && n >= (uint32_t)((*GETINT[i])(GETINT_VECWIDTH) * (*GETINT[i])(GETINT_VECWIDTH))) {
       bestPriority = (*GETINT[i])(GETINT_DFTPRIORITY);
       p->isa = i;
     }
