@@ -1,4 +1,4 @@
-//   Copyright Naoki Shibata and contributors 2010 - 2020.
+//   Copyright Naoki Shibata and contributors 2010 - 2023.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -17,10 +17,10 @@
 #define _GNU_SOURCE
 #include <unistd.h>
 #include <sys/syscall.h>
-#include <linux/random.h>
 #endif
 
 #include "sleef.h"
+#include "quaddef.h"
 #include "testerutil.h"
 
 #ifdef ENABLE_SSE2
@@ -131,34 +131,98 @@ typedef Sleef_float32x4_t_2 vfloat2;
 #define CONFIG 1
 #include "helperpower_128.h"
 #include "renamevsx.h"
-typedef Sleef___vector_double_2 vdouble2;
-typedef Sleef___vector_float_2 vfloat2;
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 
 #ifdef ENABLE_VSXNOFMA
 #define CONFIG 2
 #include "helperpower_128.h"
 #include "renamevsxnofma.h"
-typedef Sleef___vector_double_2 vdouble2;
-typedef Sleef___vector_float_2 vfloat2;
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 
-#ifdef ENABLE_ZVECTOR2
+#ifdef ENABLE_VSX3
+#define CONFIG 3
+#include "helperpower_128.h"
+#include "renamevsx3.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+
+#ifdef ENABLE_VSX3NOFMA
+#define CONFIG 4
+#include "helperpower_128.h"
+#include "renamevsx3nofma.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+
+#ifdef ENABLE_VXE
 #define CONFIG 140
 #include "helpers390x_128.h"
-#include "renamezvector2.h"
+#include "renamevxe.h"
 typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
 typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 
-#ifdef ENABLE_ZVECTOR2NOFMA
+#ifdef ENABLE_VXENOFMA
 #define CONFIG 141
 #include "helpers390x_128.h"
-#include "renamezvector2nofma.h"
+#include "renamevxenofma.h"
 typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
 typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
 #endif
 
+#ifdef ENABLE_VXE2
+#define CONFIG 150
+#include "helpers390x_128.h"
+#include "renamevxe2.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+
+#ifdef ENABLE_VXE2NOFMA
+#define CONFIG 151
+#include "helpers390x_128.h"
+#include "renamevxe2nofma.h"
+typedef Sleef_SLEEF_VECTOR_DOUBLE_2 vdouble2;
+typedef Sleef_SLEEF_VECTOR_FLOAT_2 vfloat2;
+#endif
+
+#ifdef ENABLE_RVVM1
+#define CONFIG 1
+#define ENABLE_RVV_SP
+#include "helperrvv.h"
+#include "renamervvm1.h"
+#include "sleef.h"
+#endif
+
+#ifdef ENABLE_RVVM1NOFMA
+#define CONFIG 2
+#define ENABLE_RVV_DP
+#include "helperrvv.h"
+#include "renamervvm1nofma.h"
+#include "sleef.h"
+#endif
+
+#ifdef ENABLE_RVVM2
+#define CONFIG 1
+#define ENABLE_RVV_SP
+#include "helperrvv.h"
+#include "renamervvm2.h"
+#include "sleef.h"
+#endif
+
+#ifdef ENABLE_RVVM2NOFMA
+#define CONFIG 2
+#define ENABLE_RVV_DP
+#include "helperrvv.h"
+#include "renamervvm2nofma.h"
+#include "sleef.h"
+#endif
+
 #ifdef ENABLE_PUREC_SCALAR
 #define CONFIG 1
 #include "helperpurec_scalar.h"
@@ -177,7 +241,7 @@ typedef Sleef_float_2 vfloat2;
 
 //
 
-#if !(defined(ENABLE_SVE) || defined(ENABLE_SVENOFMA))
+#if !(defined(ENABLE_SVE) || defined(ENABLE_SVENOFMA) || defined(ENABLE_RVVM1) || defined(ENABLE_RVVM1NOFMA) || defined(ENABLE_RVVM2) || defined(ENABLE_RVVM2NOFMA))
 static vfloat vf2getx_vf_vf2(vfloat2 v) { return v.x; }
 static vfloat vf2gety_vf_vf2(vfloat2 v) { return v.y; }
 #endif
@@ -1057,7 +1121,6 @@ int main(int argc,char **argv)
       mpfr_hypot(frx, frx, fry, GMP_RNDN);
 
       double u0 = countULP2sp(t = vget(xhypotf_u05(vd, vd2), e), frx);
-      double c = mpfr_get_d(frx, GMP_RNDN);
 
       if (u0 > 0.5001) {
 	printf(ISANAME " hypotf_u05 arg=%.20g, %.20g  ulp=%.20g\n", d, d2, u0);
@@ -1072,7 +1135,6 @@ int main(int argc,char **argv)
       mpfr_hypot(frx, frx, fry, GMP_RNDN);
 
       double u0 = countULP2sp(t = vget(xhypotf_u35(vd, vd2), e), frx);
-      double c = mpfr_get_d(frx, GMP_RNDN);
 
       if (u0 >= 3.5) {
 	printf(ISANAME " hypotf_u35 arg=%.20g, %.20g  ulp=%.20g\n", d, d2, u0);
@@ -1087,7 +1149,6 @@ int main(int argc,char **argv)
       mpfr_fmod(frx, frx, fry, GMP_RNDN);
 
       double u0 = countULPsp(t = vget(xfmodf(vd, vd2), e), frx);
-      long double c = mpfr_get_ld(frx, GMP_RNDN);
 
       if (fabs((double)d / d2) < 1e+38 && u0 > 0.5) {
 	printf(ISANAME " fmodf arg=%.20g, %.20g  ulp=%.20g\n", d, d2, u0);
@@ -1102,7 +1163,6 @@ int main(int argc,char **argv)
       mpfr_remainder(frx, frx, fry, GMP_RNDN);
 
       double u0 = countULPsp(t = vget(xremainderf(vd, vd2), e), frx);
-      long double c = mpfr_get_ld(frx, GMP_RNDN);
 
       if (fabs((double)d / d2) < 1e+38 && u0 > 0.5) {
 	printf(ISANAME " remainderf arg=%.20g, %.20g  ulp=%.20g\n", d, d2, u0);
@@ -1188,7 +1248,7 @@ int main(int argc,char **argv)
 
       double u0 = countULP2sp(t = vget(xerff_u1(vd), e), frx);
       
-      if (u0 > 1.0) {
+      if (u0 > 0.75) {
 	printf(ISANAME " erff_u1 arg=%.20g ulp=%.20g\n", d, u0);
 	printf("correct = %.20g, test = %.20g\n", mpfr_get_d(frx, GMP_RNDN), t);
 	fflush(stdout); ecnt++;
@@ -1234,4 +1294,5 @@ int main(int argc,char **argv)
     }
 #endif
   }
+  mpfr_clears(frw, frx, fry, frz, NULL);
 }
