@@ -1,4 +1,4 @@
-//   Copyright Naoki Shibata and contributors 2010 - 2020.
+//   Copyright Naoki Shibata and contributors 2010 - 2021.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -880,7 +880,6 @@ int main(int argc,char **argv)
       mpfr_hypot(frx, frx, fry, GMP_RNDN);
 
       double u0 = countULP2sp(t = xhypotf_u05(d, d2), frx);
-      double c = mpfr_get_d(frx, GMP_RNDN);
 
       if (u0 > 0.5001) {
 	printf("Pure C hypotf_u05 arg=%.20g, %.20g  ulp=%.20g\n", d, d2, u0);
@@ -895,7 +894,6 @@ int main(int argc,char **argv)
       mpfr_hypot(frx, frx, fry, GMP_RNDN);
 
       double u0 = countULP2sp(t = xhypotf_u35(d, d2), frx);
-      double c = mpfr_get_d(frx, GMP_RNDN);
 
       if (u0 >= 3.5) {
 	printf("Pure C hypotf_u35 arg=%.20g, %.20g  ulp=%.20g\n", d, d2, u0);
@@ -910,7 +908,6 @@ int main(int argc,char **argv)
       mpfr_fmod(frx, frx, fry, GMP_RNDN);
 
       double u0 = countULPsp(t = xfmodf(d, d2), frx);
-      long double c = mpfr_get_ld(frx, GMP_RNDN);
 
       if (fabs((double)d / d2) < 1e+38 && u0 > 0.5) {
 	printf("Pure C fmodf arg=%.20g, %.20g  ulp=%.20g\n", d, d2, u0);
@@ -925,7 +922,6 @@ int main(int argc,char **argv)
       mpfr_remainder(frx, frx, fry, GMP_RNDN);
 
       double u0 = countULPsp(t = xremainderf(d, d2), frx);
-      long double c = mpfr_get_ld(frx, GMP_RNDN);
 
       if (fabs((double)d / d2) < 1e+38 && u0 > 0.5) {
 	printf("Pure C remainderf arg=%.20g, %.20g  ulp=%.20g\n", d, d2, u0);
@@ -996,7 +992,7 @@ int main(int argc,char **argv)
 
       double u0 = countULP2sp(t = xerff_u1(d), frx);
 
-      if (u0 > 1.0) {
+      if (u0 > 0.75) {
 	printf("Pure C erff arg=%.20g ulp=%.20g\n", d, u0);
 	printf("Correct = %.20Lg, test = %.20g\n", mpfr_get_ld(frx, GMP_RNDN), t);
 	fflush(stdout); ecnt++;
@@ -1035,6 +1031,7 @@ int main(int argc,char **argv)
       }
     }
   }
+  mpfr_clears(frw, frx, fry, frz, NULL);
 
   exit(0);
 }
