@@ -1,4 +1,4 @@
-//   Copyright Naoki Shibata and contributors 2010 - 2020.
+//   Copyright Naoki Shibata and contributors 2010 - 2021.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -7,17 +7,18 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
+#include <assert.h>
 
 #include "qfuncproto.h"
 
 int main(int argc, char **argv) {
-  if (argc < 3) {
+  if (argc < 2) {
     fprintf(stderr, "Generate a header for renaming functions\n");
     fprintf(stderr, "Usage : %s <width> [<isa>]\n", argv[0]);
     fprintf(stderr, "\n");
 
     fprintf(stderr, "Generate a part of header for library functions\n");
-    fprintf(stderr, "Usage : %s <width> <vargquad type> <vargquad2 type> <vdouble type> <vfloat type> <vmask type> <vint type> <Macro to enable> [<isa>]\n", argv[0]);
+    fprintf(stderr, "Usage : %s <width> <vargquad type> <vargquad2 type> <vdouble type> <vfloat type> <vmask type> <vint type> <vint64 type> <vuint64 type> <Macro to enable> [<isa>]\n", argv[0]);
     fprintf(stderr, "\n");
 
     exit(-1);
@@ -47,12 +48,14 @@ int main(int argc, char **argv) {
     char *vargquadname = argv[2];
     char *vargquad2name = argv[3];
     char *vdoublename = argv[4];
-    char *vfloatname = argv[5];
-    char *vmaskname = argv[6];
+    //char *vfloatname = argv[5];
+    //char *vmaskname = argv[6];
     char *vintname = argv[7];
-    char *architecture = argv[8];
-    char *isaname = argc == 10 ? argv[9] : "";
-    char *isaub = argc == 10 ? "_" : "";
+    char *vint64name = argv[8];
+    char *vuint64name = argv[9];
+    char *architecture = argv[10];
+    char *isaname = argc == 12 ? argv[11] : "";
+    char *isaub = argc == 12 ? "_" : "";
 
     if (strcmp(isaname, "sve") == 0) wqp = "x";
 
@@ -63,13 +66,13 @@ int main(int argc, char **argv) {
 	switch(funcList[i].funcType) {
 	case 0:
 	  if (funcList[i].ulp >= 0) {
-	    printf("IMPORT CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   funcList[i].ulp, isaname,
 		   vargquadname);
 	  } else {
-	    printf("IMPORT CONST %s Sleef_%sq%s%s%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   isaub, isaname,
@@ -78,13 +81,13 @@ int main(int argc, char **argv) {
 	  break;
 	case 1:
 	  if (funcList[i].ulp >= 0) {
-	    printf("IMPORT CONST %s Sleef_%sq%s_u%02d%s(%s, %s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s, %s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   funcList[i].ulp, isaname,
 		   vargquadname, vargquadname);
 	  } else {
-	    printf("IMPORT CONST %s Sleef_%sq%s%s%s(%s, %s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s, %s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   isaub, isaname,
@@ -94,13 +97,13 @@ int main(int argc, char **argv) {
 	case 2:
 	case 6:
 	  if (funcList[i].ulp >= 0) {
-	    printf("IMPORT CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
 		   vargquad2name,
 		   funcList[i].name, wqp,
 		   funcList[i].ulp, isaname,
 		   vargquadname);
 	  } else {
-	    printf("IMPORT CONST %s Sleef_%sq%s%s%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s);\n",
 		   vargquad2name,
 		   funcList[i].name, wqp,
 		   isaub, isaname,
@@ -109,13 +112,13 @@ int main(int argc, char **argv) {
 	  break;
 	case 3:
 	  if (funcList[i].ulp >= 0) {
-	    printf("IMPORT CONST %s Sleef_%sq%s_u%02d%s(%s, %s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s, %s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   funcList[i].ulp, isaname,
 		   vargquadname, vintname);
 	  } else {
-	    printf("IMPORT CONST %s Sleef_%sq%s%s%s(%s, %s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s, %s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   isaub, isaname,
@@ -124,13 +127,13 @@ int main(int argc, char **argv) {
 	  break;
 	case 4:
 	  if (funcList[i].ulp >= 0) {
-	    printf("IMPORT CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
 		   vintname,
 		   funcList[i].name, wqp,
 		   funcList[i].ulp, isaname,
 		   vargquadname);
 	  } else {
-	    printf("IMPORT CONST %s Sleef_%sq%s%s%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s);\n",
 		   vintname,
 		   funcList[i].name, wqp,
 		   isaub, isaname,
@@ -139,13 +142,13 @@ int main(int argc, char **argv) {
 	  break;
 	case 5:
 	  if (funcList[i].ulp >= 0) {
-	    printf("IMPORT CONST %s Sleef_%sq%s_u%02d%s(%s, %s, %s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s, %s, %s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   funcList[i].ulp, isaname,
 		   vargquadname, vargquadname, vargquadname);
 	  } else {
-	    printf("IMPORT CONST %s Sleef_%sq%s%s%s(%s, %s, %s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s, %s, %s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   isaub, isaname,
@@ -153,20 +156,20 @@ int main(int argc, char **argv) {
 	  }
 	  break;
 	case 7:
-	  printf("IMPORT CONST int Sleef_%sq%s%s%s(int);\n", funcList[i].name, wqp, isaub, isaname);
+	  printf("SLEEF_IMPORT SLEEF_CONST int Sleef_%sq%s%s%s(int);\n", funcList[i].name, wqp, isaub, isaname);
 	  break;
 	case 8:
-	  printf("IMPORT CONST void *Sleef_%sq%s%s%s(int);\n", funcList[i].name, wqp, isaub, isaname);
+	  printf("SLEEF_IMPORT SLEEF_CONST void *Sleef_%sq%s%s%s(int);\n", funcList[i].name, wqp, isaub, isaname);
 	  break;
 	case 9:
 	  if (funcList[i].ulp >= 0) {
-	    printf("IMPORT CONST %s Sleef_%sq%s_u%02d%s(%s, %s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s, %s);\n",
 		   vintname,
 		   funcList[i].name, wqp,
 		   funcList[i].ulp, isaname,
 		   vargquadname, vargquadname);
 	  } else {
-	    printf("IMPORT CONST %s Sleef_%sq%s%s%s(%s, %s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s, %s);\n",
 		   vintname,
 		   funcList[i].name, wqp,
 		   isaub, isaname,
@@ -175,13 +178,13 @@ int main(int argc, char **argv) {
 	  break;
 	case 10:
 	  if (funcList[i].ulp >= 0) {
-	    printf("IMPORT CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
 		   vdoublename,
 		   funcList[i].name, wqp,
 		   funcList[i].ulp, isaname,
 		   vargquadname);
 	  } else {
-	    printf("IMPORT CONST %s Sleef_%sq%s%s%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s);\n",
 		   vdoublename,
 		   funcList[i].name, wqp,
 		   isaub, isaname,
@@ -190,13 +193,13 @@ int main(int argc, char **argv) {
 	  break;
 	case 11:
 	  if (funcList[i].ulp >= 0) {
-	    printf("IMPORT CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   funcList[i].ulp, isaname,
 		   vdoublename);
 	  } else {
-	    printf("IMPORT CONST %s Sleef_%sq%s%s%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   isaub, isaname,
@@ -204,33 +207,126 @@ int main(int argc, char **argv) {
 	  }
 	  break;
 	case 12:
+	  assert(funcList[i].ulp == -1);
+	  printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(Sleef_quad);\n",
+		 vargquadname,
+		 funcList[i].name, wqp,
+		 isaub, isaname);
+	  break;
+	case 13:
+	  assert(funcList[i].ulp == -1);
+	  printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s, Sleef_quad, Sleef_quad);\n",
+		 vargquadname,
+		 funcList[i].name, wqp,
+		 isaub, isaname,
+		 vintname);
+	  break;
+
+	case 14:
+	  assert(funcList[i].ulp == -1);
+	  printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(Sleef_quad *);\n",
+		 vargquadname,
+		 funcList[i].name, wqp,
+		 isaub, isaname);
+	  break;
+	case 15:
+	  assert(funcList[i].ulp == -1);
+	  printf("SLEEF_IMPORT SLEEF_CONST void Sleef_%sq%s%s%s(Sleef_quad *, %s);\n",
+		 funcList[i].name, wqp,
+		 isaub, isaname,
+		 vargquadname);
+	  break;
+	case 16:
+	  assert(funcList[i].ulp == -1);
+	  printf("SLEEF_IMPORT SLEEF_CONST Sleef_quad Sleef_%sq%s%s%s(%s, int);\n",
+		 funcList[i].name, wqp,
+		 isaub, isaname,
+		 vargquadname);
+	  break;
+	case 17:
+	  assert(funcList[i].ulp == -1);
+	  printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s, int, Sleef_quad);\n",
+		 vargquadname,
+		 funcList[i].name, wqp,
+		 isaub, isaname,
+		 vargquadname);
+	  break;
+	case 18:
 	  if (funcList[i].ulp >= 0) {
-	    printf("IMPORT CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
-		   vmaskname,
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
+		   vint64name,
 		   funcList[i].name, wqp,
 		   funcList[i].ulp, isaname,
 		   vargquadname);
 	  } else {
-	    printf("IMPORT CONST %s Sleef_%sq%s%s%s(%s);\n",
-		   vmaskname,
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s);\n",
+		   vint64name,
 		   funcList[i].name, wqp,
 		   isaub, isaname,
 		   vargquadname);
 	  }
 	  break;
-	case 13:
+	case 19:
 	  if (funcList[i].ulp >= 0) {
-	    printf("IMPORT CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   funcList[i].ulp, isaname,
-		   vmaskname);
+		   vint64name);
 	  } else {
-	    printf("IMPORT CONST %s Sleef_%sq%s%s%s(%s);\n",
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s);\n",
 		   vargquadname,
 		   funcList[i].name, wqp,
 		   isaub, isaname,
-		   vmaskname);
+		   vint64name);
+	  }
+	  break;
+	case 20:
+	  if (funcList[i].ulp >= 0) {
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
+		   vuint64name,
+		   funcList[i].name, wqp,
+		   funcList[i].ulp, isaname,
+		   vargquadname);
+	  } else {
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s);\n",
+		   vuint64name,
+		   funcList[i].name, wqp,
+		   isaub, isaname,
+		   vargquadname);
+	  }
+	  break;
+	case 21:
+	  if (funcList[i].ulp >= 0) {
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s_u%02d%s(%s);\n",
+		   vargquadname,
+		   funcList[i].name, wqp,
+		   funcList[i].ulp, isaname,
+		   vuint64name);
+	  } else {
+	    printf("SLEEF_IMPORT SLEEF_CONST %s Sleef_%sq%s%s%s(%s);\n",
+		   vargquadname,
+		   funcList[i].name, wqp,
+		   isaub, isaname,
+		   vuint64name);
+	  }
+	  break;
+	case 22:
+	  {
+	    printf("SLEEF_IMPORT %s Sleef_%sq%s%s%s(%s, %s *);\n",
+		   vargquadname,
+		   funcList[i].name, wqp,
+		   isaub, isaname,
+		   vargquadname, vintname);
+	  }
+	  break;
+	case 23:
+	  {
+	    printf("SLEEF_IMPORT %s Sleef_%sq%s%s%s(%s, %s *);\n",
+		   vargquadname,
+		   funcList[i].name, wqp,
+		   isaub, isaname,
+		   vargquadname, vargquadname);
 	  }
 	  break;
 	}
